/**************************************************************************//**
 * @file
 * @brief LED driver code for Energy Micro EFM32_G8xx_STK starter kit
 * @author Energy Micro AS
 * @version 1.0.1
 ******************************************************************************
 * @section License
 * <b>(C) Copyright 2010 Energy Micro AS, http://www.energymicro.com</b>
 ******************************************************************************
 *
 * This source code is the property of Energy Micro AS. The source and compiled
 * code may only be used on Energy Micro "EFM32" microcontrollers.
 *
 * This copyright notice may not be removed from the source code nor changed.
 *
 * DISCLAIMER OF WARRANTY/LIMITATION OF REMEDIES: Energy Micro AS has no
 * obligation to support this Software. Energy Micro AS is providing the
 * Software "AS IS", with no express or implied warranties of any kind,
 * including, but not limited to, any implied warranties of merchantability
 * or fitness for any particular purpose or warranties against infringement
 * of any proprietary rights of a third party.
 *
 * Energy Micro AS will not be liable for any consequential, incidental, or
 * special damages, or any other relief, or for any claim by any third party,
 * arising from your use of this Software.
 *
 *****************************************************************************/
#include "efm32.h"
#include "leds.h"

/**************************************************************************//**
 * @brief Light up LED
 * @param led LED number (0-3)
 *****************************************************************************/
void LED_Set(int led)
{
  GPIO->P[2].DOUT |= (1 << led);
}


/**************************************************************************//**
 * @brief Turn off LED
 * @param led LED number (0-3)
 *****************************************************************************/
void LED_Clear(int led)
{
  GPIO->P[2].DOUT &= ~(1 << led);
}


/**************************************************************************//**
 * @brief Toggle LED, switch from on to off or vice versa
 * @param led LED number (0-3)
 *****************************************************************************/
void LED_Toggle(int led)
{
  if (GPIO->P[2].DOUT & (1 << led))
  {
    GPIO->P[2].DOUT &= ~(1 << led);
  }
  else
  {
    GPIO->P[2].DOUT |= (1 << led);
  }
}


/**************************************************************************//**
 * @brief Light up LEDs according value of 4 least significat bits
 * @param value Bit pattern
 *****************************************************************************/
void LED_Value(int value)
{
  uint32_t dout;

  value &= 0xf;
  dout   = GPIO->P[2].DOUT;
  dout  &= ~(0x0000000f);
  dout  |= value;

  GPIO->P[2].DOUT = dout;
}


/**************************************************************************//**
 * @brief Initialize LED interface
 *****************************************************************************/
void LED_Init(void)
{
  GPIO_TypeDef *gpio = GPIO;
  uint32_t     gpiomode;

  /* Enable GPIO */
  CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_GPIO;

  /* Configure GPIO port C 0-3 as LED control outputs */
  gpiomode  = GPIO->P[2].MODEL;
  gpiomode &= ~(
    _GPIO_P_MODEL_MODE0_MASK |
    _GPIO_P_MODEL_MODE1_MASK |
    _GPIO_P_MODEL_MODE2_MASK |
    _GPIO_P_MODEL_MODE3_MASK);
  gpiomode |= GPIO_P_MODEL_MODE0_PUSHPULL;
  gpiomode |= GPIO_P_MODEL_MODE1_PUSHPULL;
  gpiomode |= GPIO_P_MODEL_MODE2_PUSHPULL;
  gpiomode |= GPIO_P_MODEL_MODE3_PUSHPULL;

  gpio->P[2].MODEL = gpiomode;

  /* Light them all */
  LED_Value(0xf);
}
