#include "energymodes.h"

/* If this flag is set we will disconnect the debug interface to allow
 * going down when debugger is connected. However this will disable debug
 * access, and thus can be a potential hazard if we're locked out from
 * debug completly */
#undef DISCONNECT_DEBUG_INTERFACE

#define RTCTICKFREQ (configRTCCLOCKFREQUENCY/((1<<configRTCDIVIDER)))

void EM_Enter(unsigned char em)
{
  int dmaclk, auxclk;
  /* Disable AUXHFRCO (debug clock prevents E Modes) */
  if (CMU->STATUS & CMU_STATUS_AUXHFRCOENS)
  {
    auxclk        = 1;
    CMU->OSCENCMD = CMU_OSCENCMD_AUXHFRCODIS;
  }

  /* Make sure DMA clock is running to enter EM2 (see chip errata) */
  if (CMU->HFCORECLKEN0 & CMU_HFCORECLKEN0_DMA)
  {
    dmaclk = 1;
  }
  else
  {
    CMU->HFCORECLKEN0 |= CMU_HFCORECLKEN0_DMA;
    dmaclk             = 0;
  }
  /* Disconnect debug interface (disable pull-up/down) */
#if defined(DISCONNECT_DEBUG_INTERFACE)
  GPIO->ROUTE = 0;
#endif

  if(em==1){
	//EM 1
  	SCB->SCR &= ~(1 << SCB_SCR_SLEEPDEEP_Pos);
  }else if(em==2) {
	//EM2
  	SCB->SCR |= (1 << SCB_SCR_SLEEPDEEP_Pos); 
  }else if(em ==3){
	SCB->SCR |= (1 << SCB_SCR_SLEEPDEEP_Pos); 
 	CMU->OSCENCMD = CMU_OSCENCMD_LFXODIS | CMU_OSCENCMD_LFRCODIS;

  }
 
  __WFE();
  if(em==3){
	CMU->OSCENCMD = CMU_OSCENCMD_LFXOEN | CMU_OSCENCMD_LFRCOEN;
  }
	#if configUSE_HFXO_AS_HFCLK == 1
  		CMU->OSCENCMD = CMU_OSCENCMD_HFXOEN;
		while (!(CMU->STATUS & CMU_STATUS_HFXORDY)) ;  
  		CMU->CMD=CMU_CMD_HFCLKSEL_HFXO;  
	#endif
  /* Renenable AUXHFRCO */
  if (auxclk == 1)
  {
    CMU->OSCENCMD = CMU_OSCENCMD_AUXHFRCOEN;
  }
  /* Restore DMA clock (see chip errata) */
  if (!dmaclk)
  {
    CMU->HFCORECLKEN0 &= ~CMU_HFCORECLKEN0_DMA;
  }
  
}

/*Used to compensate for rounding errors.*/
unsigned int rtcOverflows=0;
unsigned int numberOfSleeps = 0;
portTickType xSleepWhileIdle(){
		portTickType xTickCountIncrement=0;
				
		/*Make sure the eventregister is set to 0 before we interrupts are disabled.*/
		__SEV();
		__WFE();		
		portENTER_CRITICAL();
		/*Get tickcount, cant use xTaskGetTickCount() since it enables interrupts*/
		portTickType currentTick = xTaskGetTickCount();		
		unsigned char emToGoTo = ucTaskEmAllowed();
		if(emToGoTo==0)
		{
			return 0;
		}
		/*If the RTC is syncing the previously written value of COMP0, it is
		not safe to go to sleep now*/
		if(RTC->SYNCBUSY){
			return 0;
		}
		
		/* tickNextEvent is set to portMAX_DELAY. We will not try to sleep
		 longer than that. If the tick count is going to overflow before the next
		 event, we will wake up in time to handle it.*/
		portTickType tickNextEvent = portMAX_DELAY;
		
		#if configUSE_CO_ROUTINES == 1
			portTickType nextCoRoutine= xCoRoutineNextTick(currentTick);
			if(	nextCoRoutine == 0 || nextCoRoutine < currentTick){
				return 0;
			}else{
				tickNextEvent=nextCoRoutine;
			}
		#endif
		portTickType nextTask = xTaskNextTick();
		if(nextTask==0 || nextTask < currentTick){
			return 0;
		}else{
			if(nextTask<tickNextEvent){
				tickNextEvent=nextTask;
			}
		}
		portTickType ticksUntillNextEvent = tickNextEvent - currentTick;

		if(ticksUntillNextEvent<2){
			return 0;
		}

		/*Check against overflow in calculation of rtcWakeUpVal */
		if((ticksUntillNextEvent-1)>=((0xFFFFFFFF)/RTCTICKFREQ))
		{
		  ticksUntillNextEvent=((0xFFFFFFFF)/RTCTICKFREQ);
		}

		unsigned int  rtcCountBefore = RTC->CNT;
		SysTick->CTRL &= ~(1 << SysTick_CTRL_ENABLE_Pos); //Stop the systick counter	

		unsigned int rtcTickcUntillWakeup = (((ticksUntillNextEvent-1)*RTCTICKFREQ)/configTICK_RATE_HZ);
		
		#if configUSE_HFXO_AS_HFCLK == 1
			/* If HFXO is used as core clock, compensate for the long startup time of the clock.
			 * By default it takes 16384 cycles to start it up.
			 * 
			 * This could be changed to read what startup time is confgured for HFXO in the CMU
			 */
			if(rtcTickcUntillWakeup > 16384*RTCTICKFREQ/14000000){ 
				rtcTickcUntillWakeup -= 16384*RTCTICKFREQ/14000000;
			}else{
				SysTick->CTRL |= (1 << SysTick_CTRL_ENABLE_Pos); //Start the systick counter	  
				return 0;				
			}
		#endif
		unsigned int rtcWakeUpVal=rtcTickcUntillWakeup+rtcCountBefore;

		/*
		 * Guard against overflow in the RTC CNT register.
		 * Needed because we can not guarentee if it overflowed before or after we read it.
		 * 
		 * If it is less then 4 rtc ticks untill we have to wake up, we can not go to sleep.
		 * since it can take up to 3 cycles to synchronize the value into RTC->COMP0
		 */	
		if(RTC->IF & RTC_IF_OF || rtcTickcUntillWakeup <4 ){
			SysTick->CTRL |= (1 << SysTick_CTRL_ENABLE_Pos); //Start the systick counter	  
			return 0;
		}
		
		if(rtcWakeUpVal>0xFFFFFF){
			RTC->COMP0=0xFFFFFF;
		}else{
			RTC->COMP0=rtcWakeUpVal;
		}
		
		if(rtcWakeUpVal<=rtcCountBefore){
			while(1);
		}
		if(tickNextEvent<=currentTick){
			while(1);
		}
		
		EM_Enter(emToGoTo);
		unsigned int rtcCounterValue = RTC->CNT;
		SysTick->CTRL |= (1 << SysTick_CTRL_ENABLE_Pos); //Start the systick counter	
		unsigned int rtcTicksElapsed;
		
		if(RTC->IF & RTC_IF_OF){
			rtcCounterValue = RTC->CNT;		  
		  	rtcTicksElapsed = (rtcCounterValue+0xFFFFFF-rtcCountBefore);
		}else{   
			rtcTicksElapsed=(rtcCounterValue - rtcCountBefore);
		}
		RTC->COMP0=0x00FFFFFF;
		xTickCountIncrement=rtcTicksElapsed*configTICK_RATE_HZ/RTCTICKFREQ;
		/* 
		 * These next lines are to compensate for rounding errors.
		 * Since the xTickCountIncrement calculation above is done with a integer division, it will
		 * always round down. Over time, this leads to a big drift on the time.
		 *
		 * Another source of error is the fact that we do know where on the RTC clock the CNT value is read.
		 * This leads to a error of up to one period of the RTC clock each time we co to sleep. 
		 * Over time, this also leads to an error.
		 *
		 * To compensate for this xTickCountIncrement is incremented by 1 if the time in xTickCount (in task.c)		
		 * is behind the time reported by the number of RTC ticks.
		 *
		 * To make it simpler to handle overflows, 64 bit arithmetic is used at the moment.
		 *
		 * The effect of this scheme of correcting rounding errors, is that the xTickCount (in task.c)
		 * follows the RTC clock.
 		 */		
		uint64_t currentTickFromRTC = ((uint64_t) rtcOverflows<<24 | rtcCounterValue)* (uint64_t)configTICK_RATE_HZ/RTCTICKFREQ;
		uint64_t currentTickInXTickCount = (uint64_t)((uint64_t)lTaskGetNumberOfOverflows()<<32|currentTick)+xTickCountIncrement;
		if( currentTickFromRTC>currentTickInXTickCount){
			if(xTickCountIncrement+1<ticksUntillNextEvent){
				xTickCountIncrement+=1;
			}
		}else if( currentTickFromRTC<currentTickInXTickCount){
		 	 if(xTickCountIncrement>0){
				xTickCountIncrement--;
						
			 }
		}
		numberOfSleeps++;		
		if(xTickCountIncrement>=ticksUntillNextEvent){				
		  while(1);//wtf happenend?
		}
		return xTickCountIncrement;
}
