#include "inithw.h"


void GPIO_IRQInit(void);

void setupSWO(void);

void inithw(void){
	CHIP_init();
	CMU_ClockEnable(cmuClock_USART0, false);
	/*Chip errata: LFRCO/HFRCO Frequency Change during EM2/3*/
	*(volatile unsigned int*) 0x400C600CUL = 0x00020100;		
	setupSWO();
	/* Set the SEVONPEND bit in the system control register,
     * This is to make sure that interrupts wake up the core when it has 
  	 * excecuted WFE(wait for event) instruction. 
  	 */
	SCB->SCR |= (1 << SCB_SCR_SEVONPEND_Pos);
		
	#if configUSE_HFXO_AS_HFCLK == 1
    	CMU->OSCENCMD = CMU_OSCENCMD_HFXOEN;
    	while (!(CMU->STATUS & CMU_STATUS_HFXORDY)) ;  
  		CMU->CMD=CMU_CMD_HFCLKSEL_HFXO;
	#endif
	CMU->OSCENCMD = CMU_OSCENCMD_LFRCOEN;
    while (!(CMU->STATUS & CMU_STATUS_LFRCORDY)) ;  
	
	
	CMU->OSCENCMD |= CMU_OSCENCMD_LFXOEN;
  	/* Wait until LFXO clock is ready */
  	while (!(CMU->STATUS & CMU_STATUS_LFXORDY)) ;
	CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_USART2;
	CMU->HFCORECLKEN0 = CMU_HFCORECLKEN0_LE;
	CMU->HFPERCLKDIV |= CMU_HFPERCLKDIV_HFPERCLKEN;
#if configDEBUG_USE_HFCORECLK_AS_RTC_SOURCE	== 1
	CMU->LFCLKSEL = CMU_LFCLKSEL_LFA_HFCORECLKLEDIV2 | CMU_LFCLKSEL_LFB_LFXO;
#else
	CMU->LFCLKSEL = CMU_LFCLKSEL_LFA_LFXO | CMU_LFCLKSEL_LFB_LFXO;	
#endif
	DVK_SPI_init();
	vParTestInitialise();
	xSerialPortInitMinimal(0,150);
	initADC();
 	I2C_Init_TypeDef i2cInit = I2C_INIT_DEFAULT;
	I2CDRV_Init(&i2cInit);
	#if configUSE_TICKLESSIDLE == 1
		RTC_init();
	#endif
	LCD_Init(0);
 	/* Enable interrupts on pushbuttons */
  	DVK_clearInterruptFlags(0xffff);
 	DVK_enableInterrupt(BC_INTEN_JOYSTICK);
  	/* Initialize GPIO */
  	GPIO_IRQInit();
	CMU_ClockEnable(cmuClock_USART2,0);
        CMU_ClockEnable(cmuClock_I2C0,1);
        CMU_ClockEnable(cmuClock_USART0,1);
	
}

void setupSWO(void)
{
  uint32_t *dwt_ctrl = (uint32_t *) 0xE0001000;
  uint32_t *tpiu_prescaler = (uint32_t *) 0xE0040010;
  uint32_t *tpiu_protocol = (uint32_t *) 0xE00400F0;

  CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_GPIO;
  /* Enable Serial wire output pin */
  GPIO->ROUTE |= GPIO_ROUTE_SWOPEN;
  /* Set location 1 */
  GPIO->ROUTE = (GPIO->ROUTE & ~(_GPIO_ROUTE_SWLOCATION_MASK)) | GPIO_ROUTE_SWLOCATION_LOC1;
  /* Enable output on pin */
  GPIO->P[2].MODEH &= ~(_GPIO_P_MODEH_MODE15_MASK);
  GPIO->P[2].MODEH |= GPIO_P_MODEH_MODE15_PUSHPULL;
  /* Enable debug clock AUXHFRCO */
  CMU->OSCENCMD = CMU_OSCENCMD_AUXHFRCOEN;

  while(!(CMU->STATUS & CMU_STATUS_AUXHFRCORDY));

  /* Enable trace in core debug */
  CoreDebug->DHCSR |= 1;
  CoreDebug->DEMCR |= CoreDebug_DEMCR_TRCENA_Msk;

  /* Enable PC and IRQ sampling output */
  *dwt_ctrl = 0x400113FF;
  /* Set TPIU prescaler to 16. */
  *tpiu_prescaler = 0xf;
  /* Set protocol to NRZ */
  *tpiu_protocol = 2;
  /* Unlock ITM and output data */
  ITM->LAR = 0xC5ACCE55;
  ITM->TCR = 0x10009;
}

void GPIO_IRQInit(void)
{
  GPIO_TypeDef *gpio = GPIO;

  /* Configure PC14 as input */
  gpio->P[2].MODEH &= ~(_GPIO_P_MODEH_MODE14_MASK);
  gpio->P[2].MODEH |= GPIO_P_MODEH_MODE14_INPUTPULL;
  gpio->P[2].DOUT  |= (1 << 14);

  /* Set rising edge interrupt */
  gpio->EXTIFALL |= (1 << 14);

  /* Enable interrupt for PC 14 */
  gpio->EXTIPSELH &= ~(_GPIO_EXTIPSELH_EXTIPSEL14_MASK);
  gpio->EXTIPSELH |= (GPIO_EXTIPSELH_EXTIPSEL14_PORTC);
  gpio->IFC        = (1 << 14);
  gpio->IEN       |= (1 << 14);

  NVIC_SetPriority(GPIO_EVEN_IRQn,7);
  NVIC_ClearPendingIRQ(GPIO_EVEN_IRQn);
  NVIC_EnableIRQ(GPIO_EVEN_IRQn);
}

void GPIO_EVEN_IRQHandler(){
	CMU_ClockEnable(cmuClock_USART2,1);
  	DVK_clearInterruptFlags(BC_INTEN_JOYSTICK);
	CMU_ClockEnable(cmuClock_USART2,1);	
	GPIO->IFC = (1 << 14);
}