#include <stdio.h>
#include <stdlib.h>

#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "croutine.h"

#include "rtc.h"
#include "dvk.h"
#include "crflash.h"
#include "serial.h"
#include "i2cdrv.h"
#include "checktiming.h"
#include "inithw.h"
#include "measurement.h"
#include "lcdtest.h"
#include "serial.h"

#if configUSE_TICKLESSIDLE == 0
	#define vTaskCanGoToEM(x) 
#endif 
#define mainMEASUREMENT_PRIORITY	(tskIDLE_PRIORITY + 1)
#define mainCHECKTIMING_PRIORITY	(mainMEASUREMENT_PRIORITY)
#define mainRXLEUART_PRIORITY		(mainMEASUREMENT_PRIORITY)

#define mainSTACK_SIZE_FOR_TASK_USING_SPRINTF	(configMINIMAL_STACK_SIZE + 50)
xSemaphoreHandle bsemLetimerFlashLed;
xComPortHandle * portHandle;

/*
 * vTaskRxLeuart continuously reads characters from the LEUART.
 * It provides the xSerialGetString() function with a buffer big enough
 * to store the maximum size of the receive buffer.
 * 
 * When it recives a string, it sends the string back to the LEUART, adding
 * text before and a line feed after the string.
 */
static void vTaskRxLeuart( void *pvParameters );

/**************************************************************************//**
 * @brief  Main function
 *****************************************************************************/
static void vTaskCalc( void *pvParameters )
{
  portTickType time; 
  vTaskCanGoToEM(2);
  for(;;){
    vTaskDelay(40);
    time = xTaskGetTickCount(); 
    while(xTaskGetTickCount()<time+360);
  }
}

int main(void)
{
  
	inithw();
	vSemaphoreCreateBinary(bsemLetimerFlashLed);
  	//xTaskCreate( vLCDTask, "LCD", configMINIMAL_STACK_SIZE, NULL, 2, NULL );
	xTaskCreate( vMeasurementTemp, "Temperature", mainSTACK_SIZE_FOR_TASK_USING_SPRINTF, NULL,mainMEASUREMENT_PRIORITY, NULL );
	xTaskCreate( vTaskRxLeuart, "RXLEUART", configMINIMAL_STACK_SIZE, NULL,mainRXLEUART_PRIORITY, NULL );	
	xTaskCreate( vMeasurementLight, "Light", mainSTACK_SIZE_FOR_TASK_USING_SPRINTF, NULL,tskIDLE_PRIORITY, NULL );	
	//xTaskCreate( vTaskCalc, "Calc", mainSTACK_SIZE_FOR_TASK_USING_SPRINTF, NULL,tskIDLE_PRIORITY, NULL );
	//xTaskCreate( vCheckTiming, "Check", mainSTACK_SIZE_FOR_TASK_USING_SPRINTF , NULL,mainCHECKTIMING_PRIORITY, NULL );	
	#if configUSE_CO_ROUTINES == 1
		vStartFlashCoRoutines(2);
	#endif	
		
	vTaskStartScheduler();
  	return 0;
}

/*
 * This task listens to the RS223 bus, if the first character read is [0-3] it 
 * set this as the energy mode used. 
 *
 * It also echos back the string read. 
 */
static void vTaskRxLeuart( void *pvParameters )
{

/*
 * Buffer used to to give to xSerialGetString, it is important that it
 * has roomfor all the chars the function might write.
 */	
	static char rxBuffer[RXBUFFERSIZE];
	

/*
 * Buffer used to send over the LEUART.
 */
	static char txBuffer[RXBUFFERSIZE+25];
	vTaskCanGoToEM(2);
	for( ;; )
	{
	  	xSerialGetString(rxBuffer);
		switch (rxBuffer[0])
		{
		case '0': vTaskCanGoToEM(0);
		  	break;	
		case '1': vTaskCanGoToEM(1);
		  	break;
		case '2': vTaskCanGoToEM(2);
		  	break;
		case '3': vTaskCanGoToEM(3);
			break;
		default:
			break;
		}
		sprintf(txBuffer,"Received: \"%s\" \r\n",rxBuffer); 
		vSerialPutString(txBuffer,strlen(txBuffer));
	}
}

void vApplicationIdleHook( void )
{
#if configUSE_CO_ROUTINES == 1
	vCoRoutineSchedule();
#endif
}

vApplicationStackOverflowHook( xTaskHandle *pxTask, signed portCHAR *pcTaskName ){
	while(1);
}