 #include "FreeRTOS.h"
 #include "queue.h"
 #include "task.h"
 #include "semphr.h"
 
 #include "efm32_cmu.h"
 #include "efm32_gpio.h"
 #include "efm32_leuart.h"
 
 #include "serial.h"
 #include "efm32.h"
 #include "dvk.h"
#define serialLEUART_SIGFRAME '\r'
 
 static xQueueHandle xRxedChars;
 static xQueueHandle xCharsForTx;
 static xSemaphoreHandle bsemI2C;
 
 
 /**************************************************************************//**
  * @brief Intializes LEUART1 for use as an output interface, 9600-8-N-1
  *****************************************************************************/
 static void prvLEUART1_Init(void)
 {
   LEUART_TypeDef *leuart = LEUART1;
   LEUART_Init_TypeDef init;
 
   /* Enable CORE LE clock in order to access LE modules */
   CMU_ClockEnable(cmuClock_CORELE, true);
 
   /* Do not prescale clock */
   CMU_ClockDivSet(cmuClock_LEUART1, cmuClkDiv_1);
 
   /* Enable clock */
   CMU_ClockEnable(cmuClock_LEUART1, true);
 
   /* Use default location 0: TX - Pin C6, RX - Pin C7 */
   /* To avoid false start, configure output as high */
   GPIO_PinModeSet(gpioPortC, 6, gpioModePushPull, 1);
   /* Define input, no filtering */
   GPIO_PinModeSet(gpioPortC, 7, gpioModeInput, 0);
 
   /* Enable pins at default location */
   leuart->ROUTE = LEUART_ROUTE_RXPEN | LEUART_ROUTE_TXPEN;
 
   /* Configure LEUART1 */
   init.enable = leuartDisable;
   init.refFreq = 0;
   init.baudrate = 9600;
   init.databits = leuartDatabits8;
   init.parity = leuartNoParity;
   init.stopbits = leuartStopbits1;
   LEUART_Init(leuart, &init);
 
   /* Clear previous RX interrupts */
   LEUART_IntClear(LEUART1, LEUART_IF_RXDATAV);
   NVIC_ClearPendingIRQ(LEUART1_IRQn);
 
   /* Enable RX interrupts */
   LEUART_IntEnable(LEUART1, LEUART_IF_RXDATAV);
   NVIC_EnableIRQ(LEUART1_IRQn);
 
  NVIC_SetPriority(LEUART1_IRQn,7);
   LEUART_IntEnable(LEUART1, LEUART_IF_TXC);
   /* Finally enable it */
   LEUART_Enable(leuart, leuartEnable);
 }
 
 /**************************************************************************//**
 * @brief LEUART1 interrupt handler
 *****************************************************************************/
 void LEUART1_IRQHandler(void)
 {
 	LEUART_TypeDef *leuart = LEUART1;
 	portCHAR cRx;
 	portCHAR cTx;
 	portBASE_TYPE xHigherPriorityTaskWoken=pdFALSE;
 	if (leuart->IF & LEUART_IF_RXDATAV)
 	{
 		cRx = leuart->RXDATA;
 		xQueueSendFromISR(xRxedChars,&cRx,&xHigherPriorityTaskWoken);
 	}
 	if(leuart->IF & LEUART_IF_TXC ){
 		if(xQueueReceiveFromISR(xCharsForTx,&cTx,&xHigherPriorityTaskWoken) == pdTRUE){
 			leuart->TXDATA = 0xFF & cTx;
 			while(leuart->SYNCBUSY);
 		}
 		leuart->IFC = LEUART_IFC_TXC;
 	}
 	portEND_SWITCHING_ISR(xHigherPriorityTaskWoken);
 }
 
 xComPortHandle xSerialPortInitMinimal( unsigned long ulWantedBaud, unsigned portBASE_TYPE uxQueueLength )
 { 
 	xRxedChars = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );
 	xCharsForTx = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );
 	vSemaphoreCreateBinary(bsemI2C);
 	CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_USART2;
 	DVK_enablePeripheral(DVK_RS232B);
 	if(xRxedChars != 0 && xCharsForTx !=0){
 		prvLEUART1_Init();
 		return NULL;
 	}else{
 		return 0;
 	}
 
 };
 signed portBASE_TYPE xSerialPutChar( signed char cOutChar, portTickType xBlockTime ){
 	if(xQueueSend(xCharsForTx,&cOutChar,xBlockTime)==pdPASS){
 	  	if(LEUART1->STATUS & LEUART_STATUS_TXBL){
 		  if(!(LEUART1->SYNCBUSY & LEUART_SYNCBUSY_TXDATA)){
 		  	LEUART1->IFS |= LEUART_IFS_TXC;
 		  }
 		}
 		return pdPASS;
 	}
 	return pdFAIL;
 }

 
 void vSerialPutString(char * pcString, unsigned short usStringLength ){
 	signed portCHAR * pcChar = (signed char *)pcString;
 
 	xSemaphoreTake(bsemI2C,2);
 	while(*pcChar){
 		xSerialPutChar(*pcChar,2);
 		pcChar++;		
 	}
 	xSemaphoreGive(bsemI2C);
 }
  
/*
 * Returns, in pcRxString the string read. The pointer needs
 * to have room for RXBUFFERSIZE bytes.
 */
void xSerialGetString( char * pcRxedString )
{
	int i = 0;
	xQueueReceive( xRxedChars,&pcRxedString[i],portMAX_DELAY );
	while(pcRxedString[i] != serialLEUART_SIGFRAME & i < RXBUFFERSIZE){
            i++;
      	    xQueueReceive( xRxedChars,&pcRxedString[i],portMAX_DELAY );          

	}
	pcRxedString[i] = '\0';//insert a \0 at the end to make it a propper string.
}
